@@ -1149,12 +1149,19 @@ Okay:
 
 	    /* See if we're pointing at the same place as we were the last time
 	     * this command was invoked, and if this command immediately follows
-	     * another selection comand.
+	     * another selection comand.  If not, it is important to set lastUse
+	     * to NULL, otherwise trouble occurs if lastUse is an instance that
+	     * was deleted (note that this is not foolproof:  deleting an
+	     * instance followed by selecting an instance that was occupying the
+	     * same space WILL cause a crash).
 	     */
 	
 	    if (!GEO_ENCLOSE(&cmd->tx_p, &lastArea)
 		    || ((lastCommand + 1) != TxCommandNumber))
+	    {
 		samePlace = FALSE;
+		lastUse = NULL;
+	    }
 
 	    lastArea.r_xbot = cmd->tx_p.p_x - MARGIN;
 	    lastArea.r_ybot = cmd->tx_p.p_y - MARGIN;
@@ -1792,6 +1799,7 @@ cmdLabelFontFunc(label, cellUse, transform, font)
 #define SETLABEL_ROTATE		6
 #define SETLABEL_STICKY		7
 #define SETLABEL_LAYER		8
+#define SETLABEL_HELP		9
 
 void
 CmdSetLabel(w, cmd)
@@ -1799,6 +1807,7 @@ CmdSetLabel(w, cmd)
     TxCommand *cmd;
 {
     int pos = -1, font = -1, size = 0, rotate = 0, flags = 0;
+    char **msg;
     Point offset;
     TileType ttype;
     int option;
@@ -1806,7 +1815,9 @@ CmdSetLabel(w, cmd)
     Tcl_Obj *lobj;
 #endif
 
-    static char *cmdLabelYesNo[] = { "no", "false", "off", "yes", "true", "on", 0 };
+    static char *cmdLabelYesNo[] = { "no", "false", "off", "0",
+		"yes", "true", "on", "1", 0 };
+
     static char *cmdLabelSetOption[] =
     {
 	"text <text>		change/get label text",
@@ -1818,16 +1829,14 @@ CmdSetLabel(w, cmd)
 	"rotate <degrees>	change/get label rotation",
 	"sticky [true|false]	change/get sticky property",
 	"layer <type>		change/get layer type",
+	"help			print this help info",
 	NULL
     };
 
     if (cmd->tx_argc < 2 || cmd->tx_argc > 4)
-    {
-	TxError("%s <option> [<value>]\n", cmd->tx_argv[0]);
-	return;
-    }
-
-    option = Lookup(cmd->tx_argv[1], cmdLabelSetOption);
+	option = SETLABEL_HELP;
+    else
+	option = Lookup(cmd->tx_argv[1], cmdLabelSetOption);
 
     switch (option)
     {
@@ -1848,10 +1857,9 @@ CmdSetLabel(w, cmd)
 	case SETLABEL_TEXT:
 	    if (EditCellUse)
 	    {
-		SelEnumLabels(&DBAllTypeBits, FALSE, (bool *)NULL,
+		SelEnumLabels(&DBAllTypeBits, TRUE, (bool *)NULL,
 			cmdLabelTextFunc, (cmd->tx_argc == 3) ?
 			(ClientData)cmd->tx_argv[2] : (ClientData)NULL);
-		SelectTransform(&GeoIdentityTransform);
 	    }
 	    break;
 
@@ -1892,10 +1900,9 @@ CmdSetLabel(w, cmd)
 	    
 		if (EditCellUse)
 		{
-		    SelEnumLabels(&DBAllTypeBits, FALSE, (bool *)NULL,
+		    SelEnumLabels(&DBAllTypeBits, TRUE, (bool *)NULL,
 				cmdLabelFontFunc, (cmd->tx_argc == 3) ?
 				(ClientData)&font : (ClientData)NULL);
-		    SelectTransform(&GeoIdentityTransform);
 		}
 	    }
 	    break;
@@ -1908,10 +1915,9 @@ CmdSetLabel(w, cmd)
 	    }
 	    if (EditCellUse)
 	    {
-		SelEnumLabels(&DBAllTypeBits, FALSE, (bool *)NULL,
+		SelEnumLabels(&DBAllTypeBits, TRUE, (bool *)NULL,
 			cmdLabelJustFunc, (cmd->tx_argc == 3) ?
 			(ClientData)&pos : (ClientData)NULL);
-		SelectTransform(&GeoIdentityTransform);
 	    }
 	    break;
 
@@ -1924,10 +1930,9 @@ CmdSetLabel(w, cmd)
 	    }
 	    if (EditCellUse)
 	    {
-		SelEnumLabels(&DBAllTypeBits, FALSE, (bool *)NULL,
+		SelEnumLabels(&DBAllTypeBits, TRUE, (bool *)NULL,
 			cmdLabelSizeFunc, (cmd->tx_argc == 3) ?
 			(ClientData)&size : (ClientData)NULL);
-		SelectTransform(&GeoIdentityTransform);
 	    }
 	    break;
 
@@ -1953,10 +1958,9 @@ CmdSetLabel(w, cmd)
 	    }	
 	    if (EditCellUse)
 	    {
-		SelEnumLabels(&DBAllTypeBits, FALSE, (bool *)NULL,
+		SelEnumLabels(&DBAllTypeBits, TRUE, (bool *)NULL,
 			cmdLabelOffsetFunc, (cmd->tx_argc != 2) ?
 			(ClientData)&offset : (ClientData)NULL);
-		SelectTransform(&GeoIdentityTransform);
 	    }
 	    break;
 
@@ -1968,10 +1972,9 @@ CmdSetLabel(w, cmd)
 	    }
 	    if (EditCellUse)
 	    {
-		SelEnumLabels(&DBAllTypeBits, FALSE, (bool *)NULL,
+		SelEnumLabels(&DBAllTypeBits, TRUE, (bool *)NULL,
 			cmdLabelRotateFunc, (cmd->tx_argc == 3) ?
 			(ClientData)&rotate : (ClientData)NULL);
-		SelectTransform(&GeoIdentityTransform);
 	    }
 	    break;
 
@@ -1984,14 +1987,13 @@ CmdSetLabel(w, cmd)
 		    TxError("Unknown sticky option \"%s\"\n", cmd->tx_argv[2]);
 		    break;
 		}
-		flags = (option < 3) ? 0 : LABEL_STICKY;
+		flags = (option <= 3) ? 0 : LABEL_STICKY;
 	    }
 	    if (EditCellUse)
 	    {
-		SelEnumLabels(&DBAllTypeBits, FALSE, (bool *)NULL,
+		SelEnumLabels(&DBAllTypeBits, TRUE, (bool *)NULL,
 			cmdLabelStickyFunc, (cmd->tx_argc == 3) ?
 			(ClientData)&flags : (ClientData)NULL);
-		SelectTransform(&GeoIdentityTransform);
 	    }
 	    break;
 
@@ -2008,13 +2010,19 @@ CmdSetLabel(w, cmd)
 	    }
 	    if (EditCellUse)
 	    {
-		SelEnumLabels(&DBAllTypeBits, FALSE, (bool *)NULL,
+		SelEnumLabels(&DBAllTypeBits, TRUE, (bool *)NULL,
 			cmdLabelLayerFunc, (cmd->tx_argc == 3) ?
 			(ClientData)&ttype : (ClientData)NULL);
-		SelectTransform(&GeoIdentityTransform);
 	    }
 	    break;
 
+	case SETLABEL_HELP:
+	    TxError("Usage:  setlabel [option], where [option] is one of:\n");
+	    for (msg = &(cmdLabelSetOption[0]); *msg != NULL; msg++)
+	    {
+	        TxError("    %s\n", *msg);
+	    }
+	    break;
 
 	default:
 	    TxError("Unknown setlabel option \"%s\"\n", cmd->tx_argv[1]);
