@@ -949,6 +949,12 @@ dbUsePrintInfo(StartUse, who, dolist)
  *	a selected instance).  "OTHER" lists the celldef name of the
  *	instance.
  *
+ *	CellName should be referenced either to the current edit cell,
+ *	if it is not a hierarchical name;  otherwise, if it is a
+ *	hierarchical name, the instance before the last '/' is mapped
+ *	to its cellDef, and that cellDef is searched for the indicated
+ *	instance.  
+ *
  * ----------------------------------------------------------------------------
  */
 
@@ -963,9 +969,29 @@ DBUsePrint(CellName, who, dolist)
     HashEntry *entry;
     CellDef *celldef;
     CellUse *celluse;
+    char *lasthier;
 
     int dbCellUsePrintFunc();
     
+    if ((CellName != NULL) && ((lasthier = strrchr(CellName, '/')) != NULL))
+    {
+	char *prevhier;
+	*lasthier = '\0';
+	prevhier = strrchr(CellName, '/');
+	if (prevhier == NULL)
+	    prevhier = CellName;	
+	else
+	    prevhier++;
+
+	celldef = DBCellLookDef(CellName);
+	*lasthier = '/';
+    }
+    else
+    {
+	/* Referenced cellDef is the current edit def */
+	celldef = EditCellUse->cu_def;
+    }
+
     switch (who)
     {
 	case ALLCELLS:
@@ -1011,34 +1037,16 @@ DBUsePrint(CellName, who, dolist)
 	    }
 	    else
 	    {
-		SearchContext scx;
+		celluse = DBFindUse(CellName, celldef);
 
-		bzero(&scx, sizeof(SearchContext));
-		found = 0;
-
-		HashStartSearch(&hs);
-		while( (entry = HashNext(&dbCellDefTable, &hs)) != NULL)
-		{
-		    CellUse fakeuse;
-		    celldef = (CellDef *) HashGetValue(entry);
-		    if ((celldef != (CellDef *) NULL) &&
-				(!(celldef->cd_flags & CDINTERNAL)))
-		    {
-			/* DBTreeFindUse() should really take a def, not a use */
-			fakeuse.cu_def = celldef;
-			DBTreeFindUse(CellName, &fakeuse, &scx);
-			if (scx.scx_use != NULL) break;
-		    }
-		}
-
-		if (scx.scx_use == NULL)
+		if (celluse == NULL)
 		{
 		    if (!dolist)
 		        TxError("Cell %s is not currently loaded.\n", CellName);
 		}
 		else
 		{
-		    dbUsePrintInfo(scx.scx_use, who, dolist);
+		    dbUsePrintInfo(celluse, who, dolist);
 		}
 	    }
 	    break;
@@ -1822,8 +1830,13 @@ DBFindUse(id, parentDef)
     CellDef *parentDef;
 {
     HashEntry *he;
+    char *delimit;
+   
+    /* Array delimiters should be ignored */
+    if ((delimit = strrchr(id, '[')) != NULL) *delimit = '\0';
 
     he = HashLookOnly(&parentDef->cd_idHash, id);
+    if (delimit != NULL) *delimit = '[';
     if (he == NULL)
 	return (CellUse *) NULL;
 
