@@ -397,8 +397,6 @@ subcktHierVisit(use, hierName, is_top)
     EFNode *snode;
     bool hasports = FALSE;
 
-    if (def->def_flags & DEF_NODEVICES) return 0;
-
     /* Avoid generating records for circuits that have no ports.	*/
     /* These are already absorbed into the parent.  All other		*/
     /* subcircuits have at least one port marked by the EF_PORT flag.	*/
@@ -414,8 +412,10 @@ subcktHierVisit(use, hierName, is_top)
 
     if (hasports || is_top)
 	return subcktVisit(use, hierName, is_top);
-    else
+    else if (def->def_flags & DEF_NODEVICES)
 	return 0;
+    else
+	return subcktVisit(use, hierName, is_top);
 }
 
 /*
@@ -485,7 +485,22 @@ spcdevHierVisit(hc, dev, scale)
     if (dev->dev_nterm >= 2)
 	source = drain = &dev->dev_terms[1];
     if (dev->dev_nterm >= 3)
-	drain = &dev->dev_terms[2];
+    {
+        /* If any terminal is marked with attribute "D" or "S"  */
+        /* (label "D$" or "S$" at poly-diffusion interface),    */
+        /* then force order of source and drain accordingly.    */
+
+        if ((dev->dev_terms[1].dterm_attrs &&
+                !strcmp(dev->dev_terms[1].dterm_attrs, "D")) ||
+                (dev->dev_terms[2].dterm_attrs &&
+                !strcmp(dev->dev_terms[2].dterm_attrs, "S")))
+        {
+            drain = &dev->dev_terms[1];
+            source = &dev->dev_terms[2];
+        }
+        else
+            drain = &dev->dev_terms[2];
+    }
     else if (dev->dev_nterm == 1)	// Is a device with one terminal an error?
 	source = drain = &dev->dev_terms[0];
     subnode = dev->dev_subsnode;
@@ -1684,6 +1699,7 @@ esHierVisit(hc, cdata)
     char *resstr = NULL;
     DefFlagsData *dfd;
     int flags;
+    int locDoSubckt = esDoSubckt;
 
     dfd = (DefFlagsData *)cdata;
     topdef = dfd->def;
@@ -1697,11 +1713,28 @@ esHierVisit(hc, cdata)
     {
 	if (def->def_devs == NULL && def->def_uses == NULL)
 	{
-	    for (snode = (EFNode *) def->def_firstn.efnode_next;
+	    if (locDoSubckt == AUTO)
+	    {
+		/* Determine if there are ports, and don't	*/
+		/* kill the cell if it has any.			*/
+		locDoSubckt = FALSE;
+		for (snode = (EFNode *) def->def_firstn.efnode_next;
 			snode != &def->def_firstn;
 			snode = (EFNode *) snode->efnode_next)
-		snode->efnode_flags &= ~(EF_PORT | EF_SUBS_PORT);
-	    if (def != topdef) return 0;
+		    if (snode->efnode_flags & (EF_PORT | EF_SUBS_PORT))
+		    {
+			locDoSubckt = TRUE;
+			break;
+		    }
+	    }
+	    if (locDoSubckt == FALSE)
+	    {
+		for (snode = (EFNode *) def->def_firstn.efnode_next;
+			snode != &def->def_firstn;
+			snode = (EFNode *) snode->efnode_next)
+		    snode->efnode_flags &= ~(EF_PORT | EF_SUBS_PORT);
+		if (def != topdef) return 0;
+	    }
 	}
     } 
 
