@@ -422,7 +422,9 @@ CmdExtToSpice(w, cmd)
 		case IDX_TOP:
 		    if (cmd->tx_argc == 3)
 		    {
-			Tcl_SetResult(magicinterp, (esDoSubckt) ? "on" : "off", NULL);
+			Tcl_SetResult(magicinterp,
+				(esDoSubckt == 2) ? "auto" :
+				(esDoSubckt == 1) ? "on" : "off", NULL);
 			return;
 		    }
 		    break;
@@ -1309,7 +1311,7 @@ subcktVisit(use, hierName, is_top)
     EFNodeName *nodeName;
     int portorder, portmax, imp_max;
     char stmp[MAX_STR_SIZE];
-    char *instname;
+    char *instname, *subcktname;
     DevParam *plist, *pptr;
 
     if (is_top == TRUE) return 0;	/* Ignore the top-level cell */
@@ -1443,7 +1445,12 @@ subcktVisit(use, hierName, is_top)
     }    
     freeMagic(instname);
 
-    fprintf(esSpiceF, " %s\n", def->def_name);	/* subcircuit model name */
+    /* SPICE subcircuit names must begin with A-Z.  This will also be   */
+    /* enforced when writing X subcircuit calls.                        */
+    subcktname = def->def_name;
+    while (!isalpha(*subcktname)) subcktname++;
+
+    fprintf(esSpiceF, " %s\n", subcktname);	/* subcircuit model name */
     return 0;
 }
 
@@ -1510,8 +1517,14 @@ topVisit(def)
     int portorder, portmax;
     DevParam *plist, *pptr;
     char *instname;
+    char *subcktname;
 
-    fprintf(esSpiceF, ".subckt %s", def->def_name);
+    /* SPICE subcircuit names must begin with A-Z.  This will also be	*/
+    /* enforced when writing X subcircuit calls.			*/
+    subcktname = def->def_name;
+    while (!isalpha(*subcktname)) subcktname++;
+
+    fprintf(esSpiceF, ".subckt %s", subcktname);
 
     /* Note that the ports of the subcircuit will not necessarily be	*/
     /* ALL the entries in the hash table, so we have to check.		*/
@@ -1859,6 +1872,10 @@ spcWriteParams(dev, hierName, scale, l, w, sdM)
 	}
 	plist = plist->parm_next;
     }
+
+    /* Add parameters that are to be copied verbatim */
+    for (plist = dev->dev_params; plist; plist = plist->parm_next)
+	fprintf(esSpiceF, " %s", plist->parm_name);
 }
 
 /*
@@ -2002,7 +2019,22 @@ spcdevVisit(dev, hierName, scale, trans)
     if (dev->dev_nterm >= 2)
 	source = drain = &dev->dev_terms[1];
     if (dev->dev_nterm >= 3)
-	drain = &dev->dev_terms[2];
+    {
+	/* If any terminal is marked with attribute "D" or "S"	*/
+ 	/* (label "D$" or "S$" at poly-diffusion interface),	*/
+	/* then force order of source and drain accordingly.	*/
+
+	if ((dev->dev_terms[1].dterm_attrs &&
+		!strcmp(dev->dev_terms[1].dterm_attrs, "D")) ||
+		(dev->dev_terms[2].dterm_attrs &&
+		!strcmp(dev->dev_terms[2].dterm_attrs, "S")))
+	{
+	    drain = &dev->dev_terms[1];
+	    source = &dev->dev_terms[2];
+	}
+	else
+	    drain = &dev->dev_terms[2];
+    }
     subnode = dev->dev_subsnode;
 
     /* Check for minimum number of terminals. */
@@ -2806,12 +2838,10 @@ int
 spcresistVisit(hierName1, hierName2, res)
     HierName *hierName1;
     HierName *hierName2;
-    int res;
+    float res;
 {
-    res = (res + 500) / 1000;
-
-    fprintf(esSpiceF, "R%d %s %s %d\n", esResNum++, nodeSpiceName(hierName1),
-                                          nodeSpiceName(hierName2), res);
+    fprintf(esSpiceF, "R%d %s %s %g\n", esResNum++, nodeSpiceName(hierName1),
+			nodeSpiceName(hierName2), res / 1000.);
 
     return 0;
 }
