@@ -23,10 +23,11 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/extcheck/extcheck.c,v 1.2 2001/01/12 22:12:34 jsolomon Exp $";
+static char rcsid[] = "$Header: /usr/cvsroot/magic-8.0/extcheck/extcheck.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <ctype.h>
 
 #include "utils/magic.h"
@@ -75,7 +76,7 @@ main(argc, argv)
     /* Read the hierarchical description of the input circuit */
     EFReadFile(inName, FALSE, FALSE, FALSE);
     if (EFArgTech) EFTech = StrDup((char **) NULL, EFArgTech);
-    if (EFScale == 0) EFScale = 1;
+    if (EFScale == 0.0) EFScale = 1.0;
 
     /* Convert the hierarchical description to a flat one */
     EFFlatBuild(inName, EF_FLATNODES|EF_FLATCAPS|EF_FLATRESISTS);
@@ -156,8 +157,7 @@ capVisit(hn1, hn2, cap)
     double cap;
 {
     ecNumCaps++;
-    cap = (cap + 500) / 1000;
-    if (cap > (double) EFCapThreshold) ecNumThreshCaps++;
+    if ((cap / 1000.) > (double) EFCapThreshold) ecNumThreshCaps++;
     return 0;
 }
 
@@ -165,10 +165,9 @@ capVisit(hn1, hn2, cap)
 int
 resistVisit(hn1, hn2, res)
     HierName *hn1, *hn2;	/* UNUSED */
-    int res;
+    float res;
 {
     ecNumResists++;
-    res = (res + 500) / 1000;
-    if (res > EFResistThreshold) ecNumThreshResists++;
+    if ((res / 1000.) > EFResistThreshold) ecNumThreshResists++;
     return 0;
 }
