@@ -382,7 +382,7 @@ efHierDevKilled(hc, dev, prefix)
     for (n = 0; n < dev->dev_nterm; n++)
     {
 	suffix = dev->dev_terms[n].dterm_node->efnode_name->efnn_hier;
-	he = HashLookOnly(&def->def_nodes, EFHNToStr(suffix));
+	he = HashLookOnly(&efNodeHashTable, (char *)suffix);
 	if (he  && (nn = (EFNodeName *) HashGetValue(he))
 		&& (nn->efnn_node->efnode_flags & EF_KILLED))
 	    return TRUE;
@@ -710,21 +710,13 @@ EFHierVisitNodes(hc, nodeProc, cdata)
     Def *def = hc->hc_use->use_def;
     EFCapValue cap;
     int res;
-    EFNodeName *sname;
     EFNode *snode;
     HierName *hierName;
-    HashSearch hs;
-    HashEntry *he;
 
-    HashStartSearch(&hs);
-    while (he = HashNext(&def->def_nodes, &hs))
+    for (snode = (EFNode *) efNodeList.efnode_next;
+            snode != &efNodeList;
+            snode = (EFNode *) snode->efnode_next)
     {
-	sname = (EFNodeName *) HashGetValue(he);
-
-	/* The following shouldn't happen---need to investigate */
-	if (sname == (EFNodeName *)NULL) continue;
-
-	snode = sname->efnn_node;
 	res = EFNodeResist(snode);
 	cap = snode->efnode_cap;
 	hierName = (HierName *) snode->efnode_name->efnn_hier;
