@@ -187,6 +187,15 @@ efReadDef(def, dosubckt, resist, noscale, toplevel)
     name = def->def_name;
     inf = PaOpen(name, "r", ".ext", EFSearchPath, EFLibPath, &efReadFileName);
     if (inf == NULL)
+    {
+	/* Complementary to .ext file write:  If file is in a read-only	*/
+	/* directory, then .ext	file is written to CWD.			*/
+	char *proot;
+	proot = strrchr(name, '/');
+	if (proot != NULL)
+	    inf = PaOpen(proot + 1, "r", ".ext", ".", ".", &efReadFileName);
+    }
+    if (inf == NULL)
     {
 #ifdef MAGIC_WRAPPER
 	char *tclres = Tcl_Alloc(128);
@@ -393,7 +402,7 @@ readfile:
 		break;
 
 	    /*
-	     * rnode name C x y layer
+	     * rnode name R C x y layer
 	     * These are nodes resulting from resistance extraction and
 	     * so have no "intrinsic" resistance per se.
 	     */
