@@ -613,7 +613,7 @@ keys_and_buttons:
 				Tcl_SaveResult(magicinterp, &state);
 				Tcl_EvalEx(magicinterp, "history event 0", 15, 0);
 			        MacroDefine(mw->w_client, (int)'.',
-					Tcl_GetStringResult(magicinterp),
+					Tcl_GetStringResult(magicinterp), NULL,
 					FALSE);
 				Tcl_RestoreResult(magicinterp, &state);
 				break;
@@ -681,7 +681,7 @@ keys_and_buttons:
 			    *(TxBuffer + tl) = '\n';
 			    *(TxBuffer + tl + 1) = '\0';
 			    if (tl != 0) MacroDefine(mw->w_client,
-					XK_period, TxBuffer, FALSE);
+					XK_period, TxBuffer, NULL, FALSE);
 			    TxInputRedirect = TX_INPUT_NORMAL;
 			    TxSetPoint(KeyPressedEvent->x,
 					grXtransY(mw, KeyPressedEvent->y),
@@ -699,25 +699,28 @@ keys_and_buttons:
 			    TxFlushOut();
 			}
 		    }
-		    else if ((keywstate == (int)TX_LONG_CMD)
-				|| (keywstate == (int)TX_LONG_CMD2))
-		    {
-			/* Redirect input into the interpreter's terminal window */
-			TxInputRedirect = TX_INPUT_REDIRECTED;
-			if (TxTkConsole)
-			    TxSetPrompt(':');
-			else
-			{
-			    TxPrintf("\b\b: ");
-			    TxFlushOut();
-			}
-		    }
 		    else
 		    {
 			bool iMacro;
 			char *macroDef;
 
 			macroDef = MacroRetrieve(mw->w_client, keywstate, &iMacro);
+
+			/* Special handling:  An imacro beginning with ':'	*/
+			/* sets the prompt to ':' and moves to the next char.	*/
+
+			if (macroDef != NULL && *macroDef == ':' && iMacro)
+			{
+			    if (TxTkConsole)
+				TxSetPrompt(':');
+			    else
+			    {
+				TxPrintf("\b\b: ");
+				TxFlushOut();
+			    }
+			    memmove(macroDef, macroDef + 1, strlen(macroDef + 1) + 1);
+			}
+
 			macroDef = MacroSubstitute(macroDef, "%W", Tk_PathName(tkwind));
 
 			if (macroDef == NULL)
@@ -779,7 +782,7 @@ keys_and_buttons:
 		XConfigureEvent *ConfigureEvent = (XConfigureEvent*) xevent;
 		Rect	screenRect;
 		int width, height;
-		bool result;
+		bool result, need_resize;
 		    
 		width = ConfigureEvent->width;
 		height = ConfigureEvent->height;
@@ -792,11 +795,16 @@ keys_and_buttons:
             	screenRect.r_ytop = glTransYs(ConfigureEvent->y);
             	screenRect.r_ybot = glTransYs(ConfigureEvent->y + height);
 
+		need_resize = (screenRect.r_xbot != mw->w_screenArea.r_xbot ||
+			screenRect.r_xtop != mw->w_screenArea.r_xtop ||
+			screenRect.r_ybot != mw->w_screenArea.r_ybot ||
+			screenRect.r_ytop != mw->w_screenArea.r_ytop);
+
 		/* Redraw the window */
 
 		WindReframe(mw, &screenRect, FALSE, FALSE);
 		WindRedisplay(mw);
-
+		if (need_resize) (*GrCreateBackingStorePtr)(mw);
             }
             break;
 	case VisibilityNotify:
