@@ -133,11 +133,13 @@ proc magic::promptsave {type} {
 		magic::gds write $Layoutfilename
 	    }
 	  }
+      force -
       magic {
 	    set CellList [ magic::cellname list window ]
 	    if {[lsearch $CellList "(UNNAMED)"] >= 0} {
 	       set Layoutfilename [ tk_getSaveFile -filetypes \
-		   {{Magic {.mag {.mag}}} {"All files" {*}}}]
+		   {{Magic {.mag {.mag}}} {"All files" {*}}} \
+		   -title "Save cell (UNNAMED) as:" ]
 	       if {$Layoutfilename != ""} {
 		   set cellpath [file dirname $Layoutfilename]
 		   if {$cellpath == [pwd]} {
@@ -148,11 +150,66 @@ proc magic::promptsave {type} {
 		   magic::save $Layoutfilename
 	       }
 	    }
-	    magic::writeall
+	    if {$type == "force"} {
+	       magic::writeall force
+	    } else {
+	       magic::writeall
+	    }
 	  }
    }
 }
 
+# Window to prompt for a new cell
+
+proc magic::prompt_dialog { type } {
+   global Opts
+
+   if {[catch {toplevel .dialog}]} {
+      foreach child [winfo children .dialog] {
+         destroy $child
+      }
+   }
+
+   frame .dialog.titlebar
+   frame .dialog.text
+   frame .dialog.bbar
+
+   switch $type {
+      new {
+         label .dialog.titlebar.title -text "Create new cell" -foreground blue
+         button .dialog.bbar.okay -text "Okay" -command {load $Opts(cell_name); \
+		lower .dialog}
+         set Opts(cell_name) "(UNNAMED)"
+      }
+      save {
+         label .dialog.titlebar.title -text "Save cell as..." -foreground blue
+         button .dialog.bbar.okay -text "Okay" -command {save $Opts(cell_name); \
+		lower .dialog}
+         set Opts(cell_name) [cellname list window]
+      }
+   }
+   pack .dialog.titlebar.title
+
+   label .dialog.text.tlabel -text "Cell name:" -foreground brown
+   entry .dialog.text.tentry -background white -textvariable Opts(cell_name)
+
+   pack .dialog.text.tlabel -side left
+   pack .dialog.text.tentry -side left
+
+   button .dialog.bbar.cancel -text "Cancel" -command "lower .dialog"
+
+   pack .dialog.bbar.okay -side left
+   pack .dialog.bbar.cancel -side right
+
+   pack .dialog.titlebar -side top -ipadx 2 -ipady 2
+   pack .dialog.text -side top -fill both -expand true
+   pack .dialog.bbar -side top -fill x -ipadx 5
+
+   bind .dialog.text.tentry <Return> {.dialog.bbar.okay invoke}
+
+   raise .dialog
+}
+
 # Callback functions used by the DRC.
 
 proc magic::drcupdate { option } {
@@ -469,8 +526,11 @@ set Opts(netlist) 0
 set Opts(colormap) 0
 set Opts(wind3d) 0
 set Opts(crosshair) 0
+set Opts(hidelocked) 0
+set Opts(hidespecial) 0
 set Opts(toolbar) 0
 set Opts(drc) 1
+set Opts(autobuttontext) 1
 
 # Update cell and tech managers in response to a cif or gds read command
 
@@ -572,8 +632,11 @@ proc magic::cursorview {win} {
       *bypass crosshair ${olstx}i ${olsty}i
    }
 
-   set olstx [expr $olstx * $cr]
-   set olsty [expr $olsty * $cr]
+   # Use catch, because occasionally this fails on startup
+   if {[catch {
+      set olstx [expr $olstx * $cr]
+      set olsty [expr $olsty * $cr]
+   }]} {return}
 
    if {[${win} box exists]} {
       set dlst [${win} box position]
@@ -592,7 +655,6 @@ proc magic::cursorview {win} {
 proc magic::toolupdate {win {yesno "yes"} {layerlist "none"}} {
    global Winopts
 
-   # For NULL window, get current window
    if {$win == {}} {
       set win [magic::windownames]
    }
@@ -602,10 +664,11 @@ proc magic::toolupdate {win {yesno "yes"} {layerlist "none"}} {
       return
    }
 
+   set topname [winfo toplevel $win]
    set framename [winfo parent $win]
 
    # Don't do anything if toolbar is not present
-   if { $Winopts(${framename},toolbar) == 0 } { return }
+   if { $Winopts(${topname},toolbar) == 0 } { return }
 
    if {$layerlist == "none"} {
 	set layerlist $yesno
@@ -620,13 +683,17 @@ proc magic::toolupdate {win {yesno "yes"} {layerlist "none"}} {
 
    foreach layer $layerlist {
       switch $layer {
-	 none {}
+	 none {set canon ""}
+	 allSame {set canon ""}
+	 "*" {set canon ""}
+	 "$" {set canon ""}
+	 connect {set canon ""}
 	 errors {set canon $layer}
 	 subcell {set canon $layer}
 	 labels {set canon $layer}
 	 default {set canon [magic::tech layer $layer]}
       }
-      if {$layer != "none" && $canon != ""} {
+      if {$canon != ""} {
          if {$yesno == "yes"} {
 	    ${framename}.toolbar.b$canon configure -image img_$canon 
 	    ${framename}.toolbar.p$canon configure -image pale_$canon 
@@ -659,8 +726,15 @@ proc magic::maketoolimages {} {
 # Generate the toolbar for the wrapper
 
 proc magic::maketoolbar { framename } {
-
    global Opts
+   global Winopts
+
+   # Don't do anything if in suspend mode
+   set topname [winfo toplevel $framename]
+   if {[info exists Winopts(${topname},suspend)]} {
+      if { $Winopts(${topname},suspend) > 0} { return }
+   }
+
    if {$Opts(toolbar) == 0} {
       magic::maketoolimages
       set Opts(toolbar) 1
@@ -676,8 +750,17 @@ proc magic::maketoolbar { framename } {
    set win ${framename}.magic
 
    # Generate layer images and buttons for toolbar
+   if {$Opts(hidespecial) == 0} {
+       set special_layers {errors labels subcell}
+   } else {
+       set special_layers {}
+   }
 
-   set all_layers [concat {errors labels subcell} [magic::tech layer "*"]]
+   if {$Opts(hidelocked) == 0} {
+       set all_layers [concat $special_layers [magic::tech layer "*"]]
+   } else {
+       set all_layers [concat $special_layers [magic::tech unlocked]]
+   }
    foreach layername $all_layers {
       button ${framename}.toolbar.b$layername -image img_$layername -command \
 		"$win see $layername"
@@ -790,6 +873,8 @@ proc magic::techrebuild {winpath {cmdstr ""}} {
       set Opts(toolbar) 0
       maketoolbar ${framename}
       magic::techmanager init
+   } elseif {${cmdstr} == "lock" || ${cmdstr} == "unlock" || ${cmdstr} == "revert"} {
+      maketoolbar ${framename}
    }
 }
 
@@ -959,6 +1044,23 @@ proc magic::makescrollbar { fname orient win } {
    ${fname}.bar bind centre <B1-Motion> "magic::dragscroll %W %$orient $orient"
 }
 
+# Save all and quit.  If something bad happens like an attempt to
+# write cells into an unwriteable directory, then "cellname list modified"
+# will contain a list of cells, so prompt to quit with the option to cancel.
+# If there are no remaining modified and unsaved cells, then just exit.
+# Because cell "(UNNAMED)" is not written by "writeall force", if that is
+# the only modified cell, then prompt to change its name and save; then quit.
+
+proc magic::saveallandquit {} {
+   magic::promptsave force
+   set modlist [magic::cellname list modified]
+   if {$modlist == {}} {
+      magic::quit -noprompt
+   } else {
+      magic::quit
+   }
+}
+
 # Create the wrapper and open up a layout window in it.
 
 proc magic::openwrapper {{cell ""} {framename ""}} {
@@ -981,7 +1083,7 @@ proc magic::openwrapper {{cell ""} {framename ""}} {
       incr lwindow 
       set framename .layout${lwindow}
    }
-   set winname ${framename}.magic
+   set winname ${framename}.pane.top.magic
    
    toplevel $framename
    tkwait visibility $framename
@@ -991,71 +1093,85 @@ proc magic::openwrapper {{cell ""} {framename ""}} {
       catch {wm geometry ${framename} $Opts(geometry)}
    }
 
-   frame ${framename}.xscroll -height 13
-   frame ${framename}.yscroll -width 13
+   # Create a paned window top--bottom inside the top level window to accomodate
+   # a resizable command entry window at the bottom.  Sashwidth is zero by default
+   # but is resized by enabling the command entry window.
 
-   magic::makescrollbar ${framename}.xscroll x ${winname}
-   magic::makescrollbar ${framename}.yscroll y ${winname}
-   button ${framename}.zb -image $Glyph(zoom) -borderwidth 1 -command "${winname} zoom 2"
+   panedwindow ${framename}.pane -orient vertical -sashrelief groove -sashwidth 6
+
+   frame ${framename}.pane.top
+   frame ${framename}.pane.bot
+
+   set layoutframe ${framename}.pane.top
+
+   ${framename}.pane add ${framename}.pane.top
+   ${framename}.pane add ${framename}.pane.bot
+   ${framename}.pane paneconfigure ${framename}.pane.top -stretch always
+   ${framename}.pane paneconfigure ${framename}.pane.bot -hide true
+
+   pack ${framename}.pane -side top -fill both -expand true
+
+   frame ${layoutframe}.xscroll -height 13
+   frame ${layoutframe}.yscroll -width 13
+
+   magic::makescrollbar ${layoutframe}.xscroll x ${winname}
+   magic::makescrollbar ${layoutframe}.yscroll y ${winname}
+   button ${layoutframe}.zb -image $Glyph(zoom) -borderwidth 1 -command "${winname} zoom 2"
 
    # Add bindings for mouse buttons 2 and 3 to the zoom button
-   bind ${framename}.zb <Button-3> "${winname} zoom 0.5"
-   bind ${framename}.zb <Button-2> "${winname} view"
+   bind ${layoutframe}.zb <Button-3> "${winname} zoom 0.5"
+   bind ${layoutframe}.zb <Button-2> "${winname} view"
 
-   frame ${framename}.titlebar
-   label ${framename}.titlebar.caption -text "Loaded: none Editing: none Tool: box" \
+   frame ${layoutframe}.titlebar
+   label ${layoutframe}.titlebar.caption -text "Loaded: none Editing: none Tool: box" \
 	-foreground white -background sienna4 -anchor w -padx 15
-   label ${framename}.titlebar.message -text "" -foreground white \
+   label ${layoutframe}.titlebar.message -text "" -foreground white \
 	-background sienna4 -anchor w -padx 5
-   label ${framename}.titlebar.pos -text "" -foreground white \
+   label ${layoutframe}.titlebar.pos -text "" -foreground white \
 	-background sienna4 -anchor w -padx 5
 
    # Menu buttons
-   frame ${framename}.titlebar.mbuttons
+   frame ${layoutframe}.titlebar.mbuttons
 
 # File
-   menubutton ${framename}.titlebar.mbuttons.file -text File -relief raised \
-		-menu ${framename}.titlebar.mbuttons.file.toolmenu -borderwidth 2
+   menubutton ${layoutframe}.titlebar.mbuttons.file -text File -relief raised \
+		-menu ${layoutframe}.titlebar.mbuttons.file.toolmenu -borderwidth 2
 # Edit
-   menubutton ${framename}.titlebar.mbuttons.edit -text Edit -relief raised \
-		-menu ${framename}.titlebar.mbuttons.edit.toolmenu -borderwidth 2
+   menubutton ${layoutframe}.titlebar.mbuttons.edit -text Edit -relief raised \
+		-menu ${layoutframe}.titlebar.mbuttons.edit.toolmenu -borderwidth 2
 # Cell
-   menubutton ${framename}.titlebar.mbuttons.cell -text Cell -relief raised \
-		-menu ${framename}.titlebar.mbuttons.cell.toolmenu -borderwidth 2
+   menubutton ${layoutframe}.titlebar.mbuttons.cell -text Cell -relief raised \
+		-menu ${layoutframe}.titlebar.mbuttons.cell.toolmenu -borderwidth 2
 # Window
-   menubutton ${framename}.titlebar.mbuttons.win -text Window -relief raised \
-		-menu ${framename}.titlebar.mbuttons.win.toolmenu -borderwidth 2
-# Pcell
-   menubutton ${framename}.titlebar.mbuttons.pcell -text Pcell -relief raised \
-		-menu ${framename}.titlebar.mbuttons.pcell.toolmenu -borderwidth 2
+   menubutton ${layoutframe}.titlebar.mbuttons.win -text Window -relief raised \
+		-menu ${layoutframe}.titlebar.mbuttons.win.toolmenu -borderwidth 2
 # Layers
-   menubutton ${framename}.titlebar.mbuttons.layers -text Layers -relief raised \
-		-menu ${framename}.titlebar.mbuttons.layers.toolmenu -borderwidth 2
+   menubutton ${layoutframe}.titlebar.mbuttons.layers -text Layers -relief raised \
+		-menu ${layoutframe}.titlebar.mbuttons.layers.toolmenu -borderwidth 2
 # DRC
-   menubutton ${framename}.titlebar.mbuttons.drc -text Drc -relief raised \
-		-menu ${framename}.titlebar.mbuttons.drc.toolmenu -borderwidth 2
+   menubutton ${layoutframe}.titlebar.mbuttons.drc -text Drc -relief raised \
+		-menu ${layoutframe}.titlebar.mbuttons.drc.toolmenu -borderwidth 2
 # Netlist
-#   menubutton ${framename}.titlebar.mbuttons.netlist -text Neltist -relief raised \
-#		-menu ${framename}.titlebar.mbuttons.netlist.netlistmenu -borderwidth 2
+#   menubutton ${layoutframe}.titlebar.mbuttons.netlist -text Neltist -relief raised \
+#		-menu ${layoutframe}.titlebar.mbuttons.netlist.netlistmenu -borderwidth 2
 # Help
-#   menubutton ${framename}.titlebar.mbuttons.help -text Help -relief raised \
-#		-menu ${framename}.titlebar.mbuttons.help.helpmenu -borderwidth 2
+#   menubutton ${layoutframe}.titlebar.mbuttons.help -text Help -relief raised \
+#		-menu ${layoutframe}.titlebar.mbuttons.help.helpmenu -borderwidth 2
 # Options
-   menubutton ${framename}.titlebar.mbuttons.opts -text Options -relief raised \
-		-menu ${framename}.titlebar.mbuttons.opts.toolmenu -borderwidth 2
-   pack ${framename}.titlebar.mbuttons.file   -side left
-   pack ${framename}.titlebar.mbuttons.edit   -side left
-   pack ${framename}.titlebar.mbuttons.cell   -side left
-   pack ${framename}.titlebar.mbuttons.win    -side left
-   pack ${framename}.titlebar.mbuttons.pcell  -side left
-   pack ${framename}.titlebar.mbuttons.layers -side left
-   pack ${framename}.titlebar.mbuttons.drc    -side left
-#   pack ${framename}.titlebar.mbuttons.netlist -side left
-#   pack ${framename}.titlebar.mbuttons.help    -side left
-   pack ${framename}.titlebar.mbuttons.opts    -side left
+   menubutton ${layoutframe}.titlebar.mbuttons.opts -text Options -relief raised \
+		-menu ${layoutframe}.titlebar.mbuttons.opts.toolmenu -borderwidth 2
+   pack ${layoutframe}.titlebar.mbuttons.file   -side left
+   pack ${layoutframe}.titlebar.mbuttons.edit   -side left
+   pack ${layoutframe}.titlebar.mbuttons.cell   -side left
+   pack ${layoutframe}.titlebar.mbuttons.win    -side left
+   pack ${layoutframe}.titlebar.mbuttons.layers -side left
+   pack ${layoutframe}.titlebar.mbuttons.drc    -side left
+#   pack ${layoutframe}.titlebar.mbuttons.netlist -side left
+#   pack ${layoutframe}.titlebar.mbuttons.help    -side left
+   pack ${layoutframe}.titlebar.mbuttons.opts    -side left
 
    # DRC status button
-   checkbutton ${framename}.titlebar.drcbutton -text "DRC" -anchor w \
+   checkbutton ${layoutframe}.titlebar.drcbutton -text "DRC" -anchor w \
 	-borderwidth 2 -variable Opts(drc) \
 	-foreground white -background sienna4 -selectcolor green \
 	-command [subst { if { \$Opts(drc) } { drc on } else { drc off } }]
@@ -1065,28 +1181,28 @@ proc magic::openwrapper {{cell ""} {framename ""}} {
    # Create toolbar frame.  Make sure it has the same visual and depth as
    # the layout window, so there will be no problem using the GCs from the
    # layout window to paint into the toolbar.
-   frame ${framename}.toolbar \
+   frame ${layoutframe}.toolbar \
 	-visual "[winfo visual ${winname}] [winfo depth ${winname}]"
 
    # Repaint to magic colors
-   magic::repaintwrapper ${framename}
+   magic::repaintwrapper ${layoutframe}
 
-   grid ${framename}.titlebar -row 0 -column 0 -columnspan 3 -sticky news
-   grid ${framename}.yscroll -row 1 -column 0 -sticky ns
+   grid ${layoutframe}.titlebar -row 0 -column 0 -columnspan 3 -sticky news
+   grid ${layoutframe}.yscroll -row 1 -column 0 -sticky ns
    grid $winname -row 1 -column 1 -sticky news
-   grid ${framename}.zb -row 2 -column 0
-   grid ${framename}.xscroll -row 2 -column 1 -sticky ew
+   grid ${layoutframe}.zb -row 2 -column 0
+   grid ${layoutframe}.xscroll -row 2 -column 1 -sticky ew
    # The toolbar is not attached by default
 
-   grid rowconfigure ${framename} 1 -weight 1
-   grid columnconfigure ${framename} 1 -weight 1
+   grid rowconfigure ${layoutframe} 1 -weight 1
+   grid columnconfigure ${layoutframe} 1 -weight 1
 
-   grid ${framename}.titlebar.mbuttons -row 0 -column 0 -sticky news
-   grid ${framename}.titlebar.drcbutton -row 0 -column 1 -sticky news
-   grid ${framename}.titlebar.caption -row 0 -column 2 -sticky news
-   grid ${framename}.titlebar.message -row 0 -column 3 -sticky news
-   grid ${framename}.titlebar.pos -row 0 -column 4 -sticky news
-   grid columnconfigure ${framename}.titlebar 2 -weight 1
+   grid ${layoutframe}.titlebar.mbuttons -row 0 -column 0 -sticky news
+   grid ${layoutframe}.titlebar.drcbutton -row 0 -column 1 -sticky news
+   grid ${layoutframe}.titlebar.caption -row 0 -column 2 -sticky news
+   grid ${layoutframe}.titlebar.message -row 0 -column 3 -sticky news
+   grid ${layoutframe}.titlebar.pos -row 0 -column 4 -sticky news
+   grid columnconfigure ${layoutframe}.titlebar 2 -weight 1
 
    bind $winname <Enter> "focus %W ; set Opts(focus) $framename"
 
@@ -1105,153 +1221,152 @@ proc magic::openwrapper {{cell ""} {framename ""}} {
 # #################################
 # File
 # #################################
-   set m [menu ${framename}.titlebar.mbuttons.file.toolmenu -tearoff 0]
-   $m add command -label "Open...   "        -command {magic::promptload magic}
-   $m add command -label "Save      "        -command {magic::save }
-   # $m add command -label "Save      "        -command {magic::promptsave magic}
-   $m add command -label "Save as..."        -command {echo "not implemented"}
-   $m add command -label "Save selection..." -command {echo "not implemented"}
+   set m [menu ${layoutframe}.titlebar.mbuttons.file.toolmenu -tearoff 0]
+   $m add command -label "Open..."      -command {magic::promptload magic}
+   # $m add command -label "Save"       -command {magic::save }
+   $m add command -label "Save..."      -command {magic::promptsave magic}
+   # $m add command -label "Save as..." -command {echo "not implemented"}
+   # $m add command -label "Save selection..." -command {echo "not implemented"}
    $m add separator
-   $m add command -label "Flush changes "    -command {magic::flush}
-   $m add command -label "Delete "           -command {magic::cellname delete [magic::cellname list window]}
+   $m add command -label "Flush changes" -command {magic::flush}
    $m add separator
-   #$m add command -label "Read CIF" -command {magic::promptload cif}
-   $m add command -label "Read GDS"          -command {magic::promptload gds}
-   #$m add separator
-   #$m add command -label "Write CIF" -command {magic::promptsave cif}
-   $m add command -label "Write GDS"         -command {magic::promptsave gds}
+   # $m add command -label "Read CIF" -command {magic::promptload cif}
+   $m add command -label "Read GDS"    -command {magic::promptload gds}
+   # $m add separator
+   # $m add command -label "Write CIF" -command {magic::promptsave cif}
+   $m add command -label "Write GDS"   -command {magic::promptsave gds}
+   # $m add separator
+   # $m add command -label "Print..."  -command {echo "not implemented"}
    $m add separator
-   $m add command -label "Print..."          -command { echo "not implemented" }
-   $m add separator
-   $m add command -label "Save All and Quit" -command {magic::writeall force ; if {[magic::cellname list modified] == {}} {magic::quit -noprompt} else {magic::quit} }
+   $m add command -label "Save All and Quit" -command {magic::saveallandquit}
    $m add command -label "Quit"              -command {magic::quit}
 
 # #################################
 # Edit
 # #################################
 
-   set m [menu ${framename}.titlebar.mbuttons.edit.toolmenu -tearoff 0]
-   $m add command -label "Cut              " -command { echo "not implemented" }
-   $m add command -label "Copy             " -command { echo "not implemented" }
-   $m add command -label "Paste            " -command { echo "not implemented" }
-   $m add command -label "Delete           " -command { delete }
+   set m [menu ${layoutframe}.titlebar.mbuttons.edit.toolmenu -tearoff 0]
+   # $m add command -label "Cut" -command {echo "not implemented"}
+   # $m add command -label "Copy" -command {echo "not implemented"}
+   # $m add command -label "Paste" -command {echo "not implemented"}
+   $m add command -label "Delete" -command {delete}
    $m add separator
-   $m add command -label "Select Area      " -command { select area }
-   $m add command -label "Select Clear     " -command { select clear }
+   $m add command -label "Select Area" -command {select area}
+   $m add command -label "Select Clear" -command {select clear}
    $m add separator
-   $m add command -label "Undo          (u)" -command {magic::undo}
-   $m add command -label "Redo          (U)" -command {magic::redo}
-   $m add command -label "Repeat Last      " -command { echo "not implemented" }
+   $m add command -label "Undo" -command {magic::undo}
+   $m add command -label "Redo" -command {magic::redo}
+   # $m add command -label "Repeat Last" -command {echo "not implemented"}
    $m add separator
-   $m add command -label "Rotate 90 degree " -command {magic::clock}
-   $m add command -label "Mirror   Up/Down "  -command {magic::upsidedown}
+   $m add command -label "Rotate 90 degree" -command {magic::clock}
+   $m add command -label "Mirror   Up/Down"  -command {magic::upsidedown}
    $m add command -label "Mirror Left/Right" -command {magic::sideways}
    $m add separator
-   $m add command -label "Move Right       " -command {move right 1}
-   $m add command -label "Move Left        " -command {move left  1}
-   $m add command -label "Move Up          " -command {move up    1}
-   $m add command -label "Move Down        " -command {move down  1}
+   $m add command -label "Move Right" -command {move right 1}
+   $m add command -label "Move Left" -command {move left 1}
+   $m add command -label "Move Up" -command {move up 1}
+   $m add command -label "Move Down" -command {move down 1}
    $m add separator
-   $m add command -label "Stretch Right    " -command { stretch right 1}
-   $m add command -label "Stretch Left     " -command { stretch left  1}
-   $m add command -label "Stretch Up       " -command { stretch up    1}
-   $m add command -label "Stretch Down     " -command { stretch down  1}
+   $m add command -label "Stretch Right" -command {stretch right 1}
+   $m add command -label "Stretch Left" -command {stretch left 1}
+   $m add command -label "Stretch Up" -command {stretch up 1}
+   $m add command -label "Stretch Down" -command {stretch down 1}
    $m add separator
    $m add command -label "Text ..." \
-		-command [subst { magic::update_texthelper; \
-		wm deiconify .texthelper ; raise .texthelper } ]
+		-command [subst {magic::update_texthelper; \
+		wm deiconify .texthelper ; raise .texthelper}]
 
 # #################################
 # Cell
 # #################################
-   set m [menu ${framename}.titlebar.mbuttons.cell.toolmenu -tearoff 0]
-   $m add command -label "Select           " -command {magic::select cell}
-   $m add command -label "Place Instance   " -command {magic::promptload getcell}
-   $m add command -label "Rename           " -command {echo "not implemented" }
+   set m [menu ${layoutframe}.titlebar.mbuttons.cell.toolmenu -tearoff 0]
+   $m add command -label "New..." -command {magic::prompt_dialog new}
+   $m add command -label "Save as..." -command {magic::prompt_dialog save}
+   $m add command -label "Select" -command {magic::select cell}
+   $m add command -label "Place Instance" -command {magic::promptload getcell}
+   # $m add command -label "Rename" -command {echo "not implemented"}
    $m add separator
-   $m add command -label "Down hierarchy   " -command {magic::pushstack}
-   $m add command -label "Up   hierarchy   " -command {magic::popstack}
+   $m add command -label "Down hierarchy" -command {magic::pushstack}
+   $m add command -label "Up   hierarchy" -command {magic::popstack}
    $m add separator
-   $m add command -label "Edit             " -command {magic::edit }
+   $m add command -label "Edit" -command {magic::edit}
    $m add separator
-   $m add command -label "Expand Toggle (/)" -command {magic::expand toggle}
-   $m add command -label "Expand        (x)" -command {magic::expand }
-   $m add command -label "Unexpand      (X)" -command {magic::unexpand }
+   $m add command -label "Delete" -command \
+		{magic::cellname delete [magic::cellname list window]}
    $m add separator
-   $m add command -label "Lock   Cell      " -command {magic::instance lock}
-   $m add command -label "Unlock Cell      " -command {magic::instance unlock}
+   $m add command -label "Expand Toggle" -command {magic::expand toggle}
+   $m add command -label "Expand" -command {magic::expand}
+   $m add command -label "Unexpand" -command {magic::unexpand}
+   $m add separator
+   $m add command -label "Lock   Cell" -command {magic::instance lock}
+   $m add command -label "Unlock Cell" -command {magic::instance unlock}
 
 # #################################
 # Window
 # #################################
-   set m [menu ${framename}.titlebar.mbuttons.win.toolmenu -tearoff 0]
+   set m [menu ${layoutframe}.titlebar.mbuttons.win.toolmenu -tearoff 0]
+   $m add command -label "Clone" -command \
+		{magic::openwrapper [magic::cellname list window]}
    $m add command -label "New" -command "magic::openwrapper"
-   # $m add command -label "New" -command "magic::openwrapper \
-		# [${winname} cellname list window]"
-   $m add command -label "Close"             -command "magic::closewrapper ${framename}"
+   $m add command -label "Set Editable" -command \
+		"pushbox ; select top cell ; edit ; popbox"
+   $m add command -label "Close" -command "magic::closewrapper ${framename}"
    $m add separator
-   $m add command -label "Full          (f)" -command {magic::view }
-   $m add command -label "Redraw           " -command {}
-   $m add command -label "Zoom Out      (Z)" -command {magic::zoom 2}
-   $m add command -label "Zoom In          " -command {magic::zoom 0.5}
-   $m add command -label "Zoom Box      (z)" -command {magic::findbox zoom}
+   $m add command -label "Full View" -command {magic::view}
+   $m add command -label "Redraw" -command {magic::redraw}
+   $m add command -label "Zoom Out" -command {magic::zoom 2}
+   $m add command -label "Zoom In" -command {magic::zoom 0.5}
+   $m add command -label "Zoom Box" -command {magic::findbox zoom}
    $m add separator
-   $m add command -label "Pan Left         " -command {}
-   $m add command -label "Pan Right        " -command {}
-   $m add command -label "Pan Up           " -command {}
-   $m add command -label "Pan Down         " -command {}
+   $m add command -label "Grid on" -command {magic::grid on}
+   $m add command -label "Grid off" -command {magic::grid off}
+   $m add command -label "Snap-to-grid on" -command {magic::snap on}
+   $m add command -label "Snap-to-grid off" -command {magic::snap off}
+   $m add command -label "Measure box" -command {magic::box }
    $m add separator
-   $m add command -label "Grid on          " -command {magic::grid on  }
-   $m add command -label "Grid off         " -command {magic::grif off }
-   $m add command -label "Snap to grid on  " -command {magic::snap on  }
-   $m add command -label "Snap to grid off " -command {magic::snap off }
-   $m add command -label "Measure box      " -command {magic::box }
-   $m add separator
-   $m add command -label "Set grid 0.05um  " -command {magic::grid 0.05um}
-   $m add command -label "Set grid 0.10um  " -command {magic::grid 0.10um}
-   $m add command -label "Set grid 0.50um  " -command {magic::grid 0.50um}
-   $m add command -label "Set grid 1.00um  " -command {magic::grid 1.00um}
-   $m add command -label "Set grid 5.00um  " -command {magic::grid 5.00um}
-   $m add command -label "Set grid 10.0um  " -command {magic::grid 10.0um}
-   $m add command -label "Set grid ....    " -command {echo "Not implemented"}
-
-# #################################
-# Pcells
-# #################################
-   set m [menu ${framename}.titlebar.mbuttons.pcell.toolmenu -tearoff 0]
-   # $m add command -label "Create Pcell..." -command {}
+   $m add command -label "Set grid 0.05um" -command {magic::grid 0.05um}
+   $m add command -label "Set grid 0.10um" -command {magic::grid 0.10um}
+   $m add command -label "Set grid 0.50um" -command {magic::grid 0.50um}
+   $m add command -label "Set grid 1.00um" -command {magic::grid 1.00um}
+   $m add command -label "Set grid 5.00um" -command {magic::grid 5.00um}
+   $m add command -label "Set grid 10.0um" -command {magic::grid 10.0um}
+   # $m add command -label "Set grid ..." -command {echo "not implemented"}
 
 # #################################
 # Layers
 # #################################
-   set m [menu ${framename}.titlebar.mbuttons.layers.toolmenu -tearoff 0]
-   $m add command -label "Protect Base Layers" -command {magic::tech revert  }
+   set m [menu ${layoutframe}.titlebar.mbuttons.layers.toolmenu -tearoff 0]
+   $m add command -label "Protect Base Layers" -command {magic::tech revert}
    $m add command -label "Unlock  Base Layers" -command {magic::tech unlock *}
    $m add separator
-   $m add command -label "Clear Feedback"      -command {magic::feedback clear}
+   $m add command -label "Clear Feedback" -command {magic::feedback clear}
    $m add separator
 
 # #################################
 # DRC
 # #################################
-   set m [menu ${framename}.titlebar.mbuttons.drc.toolmenu -tearoff 0]
-   $m add command -label "DRC On" -command {}
-   $m add command -label "DRC Off" -command {}
+   set m [menu ${layoutframe}.titlebar.mbuttons.drc.toolmenu -tearoff 0]
+   # $m add command -label "DRC On" -command {drc on}
+   # $m add command -label "DRC Off" -command {drc off}
+   # $m add separator
+   $m add command -label "DRC update" -command {drc check; drc why}
+   $m add command -label "DRC report" -command {drc why}
+   $m add command -label "DRC Find next error" -command {drc find; findbox zoom}
    $m add separator
-   $m add command -label "DRC update" -command { drc check ; drc why}
-   $m add command -label "DRC report" -command { drc why }
-   $m add command -label "DRC Find next error" -command { drc find ; findbox zoom }
-   $m add separator
-   $m add command -label "DRC Fast"     -command {drc style drc(fast) }
-   $m add command -label "DRC Complete" -command {drc style drc(full) }
+   $m add command -label "DRC Fast"     -command {drc style drc(fast)}
+   $m add command -label "DRC Complete" -command {drc style drc(full)}
 
-   set m [menu ${framename}.titlebar.mbuttons.opts.toolmenu -tearoff 0]
+   set m [menu ${layoutframe}.titlebar.mbuttons.opts.toolmenu -tearoff 0]
    $m add check -label "Toolbar" -variable Winopts(${framename},toolbar) \
 	-command [subst {if { \$Winopts(${framename},toolbar) } { \
-		magic::maketoolbar ${framename} ; \
-		grid ${framename}.toolbar -row 1 -column 2 -rowspan 2 -sticky new ; \
+		magic::maketoolbar ${layoutframe} ; \
+		grid ${layoutframe}.toolbar -row 1 -column 2 -rowspan 2 -sticky new ; \
 		} else { \
-		grid forget ${framename}.toolbar } }]
+		grid forget ${layoutframe}.toolbar } }]
+
+   $m add check -label "Toolbar Hide Locked" \
+	-variable Opts(hidelocked) \
+	-command "magic::maketoolbar ${layoutframe}"
 
    .winmenu add radio -label ${framename} -variable Opts(target) -value ${winname}
    if {$tk_version >= 8.5} {
@@ -1317,8 +1432,8 @@ proc magic::openwrapper {{cell ""} {framename ""}} {
 
    # If the toolbar is turned on, invoke the toolbar button
    if { $Winopts(${framename},toolbar) == 1} {
-      magic::maketoolbar ${framename}
-      grid ${framename}.toolbar -row 1 -column 2 -rowspan 2 -sticky new
+      magic::maketoolbar ${layoutframe}
+      grid ${layoutframe}.toolbar -row 1 -column 2 -rowspan 2 -sticky new
    }
 
    # Remove "open" and "close" macros so they don't generate non-GUI
@@ -1349,6 +1464,14 @@ proc magic::openwrapper {{cell ""} {framename ""}} {
    # If the variable $Winopts(callback) is defined, then attempt to execute it.
    catch {eval $Winopts(${framename}, callback)}
 
+   # Since one purpose of the window callback is to customize the menus,
+   # run the automatic generation of accelerator key text at the end.
+   # This can be subverted by setting Opts(autobuttontext) to 0, e.g.,
+   # to put it at the top of the Winopts callback and then generate
+   # override values for specific buttons.
+   if {$Opts(autobuttontext)} {
+      catch {magic::button_auto_bind_text $layoutframe}
+   }
    return ${winname}
 }
 
@@ -1361,7 +1484,8 @@ proc magic::closewrapper { framename } {
    # Remove this window from the target list in .winmenu
    # (used by, e.g., cellmanager)
 
-   if { $Opts(target) == "${framename}.magic" } {
+   set layoutframe ${framename}.pane.top
+   if { $Opts(target) == "${layoutframe}.magic" } {
       set Opts(target) "default"
       if {$tk_version >= 8.5} {
 	 if {![catch {wm state .cellmgr}]} {
@@ -1373,7 +1497,7 @@ proc magic::closewrapper { framename } {
    set idx [.winmenu index $framename]
    .winmenu delete $idx
 
-   ${framename}.magic magic::closewindow
+   ${layoutframe}.magic magic::closewindow
    destroy $framename
 }
 
@@ -1381,22 +1505,42 @@ proc magic::closewrapper { framename } {
 # a wrapper window (rudimentary functionality---incomplete)
 
 proc magic::addcommandentry { framename } {
-   if {![winfo exists ${framename}.eval]} {
-      tkshell::YScrolled_Text ${framename}.eval -height 5
-      tkshell::MakeEvaluator ${framename}.eval.text \
-		"${framename}> " "${framename}.magic "
-      tkshell::MainInit
+   set commandframe ${framename}.pane.bot
+   if {![winfo exists ${commandframe}.eval]} {
+      tkshell::YScrolled_Text ${commandframe}.eval -height 5
+      tkshell::MakeEvaluator ${commandframe}.eval.text \
+		"${framename}>" ${framename}.pane.top.magic
+      pack ${commandframe}.eval -side top -fill both -expand true
+      ${framename}.pane paneconfigure ${framename}.pane.bot -stretch never
+      ${framename}.pane paneconfigure ${framename}.pane.bot -minsize 50
    }
-   set rc [grid size ${framename}]  
-   set cols [lindex $rc 0]
-   grid ${framename}.eval -row 3 -column 0 -columnspan $cols -sticky ew
-   bind ${framename}.eval.text <Enter> {focus %W}
+   set entercmd [bind ${framename}.pane.top.magic <Enter>]
+   set bindstr "$entercmd ; macro XK_colon \"focus ${commandframe}.eval.text\" ;\
+		alias puts tkshell::PutsTkShell ; set Opts(redirect) 1"
+   bind ${commandframe}.eval <Enter> \
+	"focus ${commandframe}.eval.text ; set Opts(focus) $framename ;\
+	catch {unset Opts(redirect)}"
+   bind ${framename}.pane.top.magic <Enter> $bindstr
+   # Make command entry window visible
+   ${framename}.pane paneconfigure ${framename}.pane.bot -hide false
 }
 
 # Remove the command entry window from the bottom of a frame.
 
 proc magic::deletecommandentry { framename } {
-   grid forget ${framename}.eval
+   set commandframe ${framename}.pane.bot
+   ::grid forget ${commandframe}.eval
+   # Remove the last bindings for <Enter>
+   set bindstr [bind ${framename}.pane.top.magic <Enter>]
+   set i [string first "; macro" $bindstr]
+   set bindstr [string range $bindstr 0 $i-1]
+   bind ${framename}.pane.top.magic <Enter> $bindstr
+   # Restore the keybinding for colon
+   imacro XK_colon ":"
+   # Restore the alias for "puts"
+   alias puts ::tkcon_puts
+   # Make command entry window invisible
+   ${framename}.pane paneconfigure ${framename}.pane.bot -hide true
 }
 
 namespace import magic::openwrapper
@@ -1435,37 +1579,3 @@ proc magic::render3d {{cell ""}} {
    bind .render.magic <Enter> {focus %W}
 }
 
-######################################################################
-# Visual Menu Helper Functions - Bertrand Irissou
-######################################################################
-
-###
-###  Add a menu button to the Magic wrapper window for the toolkit
-###
-proc magic::add_menu {framename toolname button_text } {
-   menubutton ${framename}.titlebar.mbuttons.${toolname} \
-		-text $button_text \
-		-relief raised \
-		-menu ${framename}.titlebar.mbuttons.${toolname}.toolmenu \
-		-borderwidth 2
-
-   menu ${framename}.titlebar.mbuttons.${toolname}.toolmenu -tearoff 0
-   pack ${framename}.titlebar.mbuttons.${toolname} -side left
-}
-
-###
-###  Add a command item to the toolkit menu
-###
-proc magic::add_menu_command {framename toolname button_text command} {
-   set m ${framename}.titlebar.mbuttons.${toolname}.toolmenu
-   $m add command -label "$button_text" -command $command 
-}
-
-###
-### Add a separator to a menu
-###
-proc magic::add_menu_separator {framename toolname} {
-   set m ${framename}.titlebar.mbuttons.${toolname}.toolmenu
-   $m add separator
-}
-
