@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/windows/windCmdSZ.c,v 1.2 2006/04/15 01:05:02 anonymous Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/windows/windCmdSZ.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -629,6 +629,8 @@ windUndoCmd(w, cmd)
     if (cmd->tx_argc > 3)
     {
 	TxError("Usage: undo [count]\n");
+	TxError("       undo print [count]\n");
+	TxError("       undo enable|disable\n");
 	return;
     }
     else if (cmd->tx_argc == 3)
@@ -654,7 +656,13 @@ windUndoCmd(w, cmd)
     {
 	if (!StrIsInt(cmd->tx_argv[1]))
 	{
-	    TxError("Option must be a count (integer)\n");
+	    if (!strcmp(cmd->tx_argv[1], "enable"))
+		UndoEnable();
+	    else if (!strcmp(cmd->tx_argv[1], "disable"))
+		UndoDisable();
+	    else
+		TxError("Option must be a count (integer)\n");
+	    return;
 	}
 	count = atoi(cmd->tx_argv[1]);
 	if (count < 0)
@@ -835,6 +843,34 @@ windViewCmd(w, cmd)
 	    Tcl_SetObjResult(magicinterp, listxy);
 #endif
 	}
+	else
+	{
+	    char *sptr, *pptr;
+	    Rect r;
+
+	    // Parse out coordinates where all coordinates have been
+	    // put into a single string argument, as happens when the
+	    // coordinates are a Tcl list, e.g., from "[box values]"
+
+	    sptr = cmd->tx_argv[1];
+	    if ((pptr = strchr(sptr, ' ')) == NULL) return;
+	    *pptr++ = '\0';
+	    r.r_xbot = cmdParseCoord(w, sptr, FALSE, TRUE);
+
+	    sptr = pptr;
+	    if ((pptr = strchr(sptr, ' ')) == NULL) return;
+	    *pptr++ = '\0';
+	    r.r_ybot = cmdParseCoord(w, sptr, FALSE, TRUE);
+
+	    sptr = pptr;
+	    if ((pptr = strchr(sptr, ' ')) == NULL) return;
+	    *pptr++ = '\0';
+	    r.r_xtop = cmdParseCoord(w, sptr, FALSE, TRUE);
+	    r.r_ytop = cmdParseCoord(w, pptr, FALSE, TRUE);
+
+	    /* Redisplay */
+	    WindMove(w, &r);
+	}
     }
     else if (cmd->tx_argc == 5)
     {
@@ -1019,7 +1055,7 @@ windSendCmd(w, cmd)
     }
     newcmd.tx_wid = WIND_UNKNOWN_WINDOW;
     if (toWindow != NULL) newcmd.tx_wid = toWindow->w_wid;
-    (void) WindSendCommand(toWindow, &newcmd);
+    (void) WindSendCommand(toWindow, &newcmd, FALSE);
     return;
 
     usage:
